==> coordinax-0.20.1-py3-none-any.whl: coordinax/_src/vectors/d1/register_vectorapi.py <==
"""Representation of coordinates in different systems."""

__all__: list[str] = []

from typing import Any

from plum import dispatch

from .base import AbstractAcc1D, AbstractPos1D, AbstractVel1D
from .cartesian import CartesianAcc1D, CartesianPos1D, CartesianVel1D
from .radial import RadialAcc, RadialPos, RadialVel
from coordinax._src.vectors.base_pos import AbstractPos
from coordinax._src.vectors.base_vel import AbstractVel

# =============================================================================
# 1D


@dispatch
def vconvert(
    target: type[AbstractPos1D], current: AbstractPos1D, /, **kwargs: Any
) -> AbstractPos1D:
    """AbstractPos1D -> Cartesian1D -> AbstractPos1D.

    This is the base case for the transformation of 1D vectors.
    """
    cart1d = vconvert(CartesianPos1D, current)
    return vconvert(target, cart1d)


# TODO: use multi, with precedence
@dispatch(precedence=1)
def vconvert(
    target: type[CartesianPos1D], current: CartesianPos1D, /, **kwargs: Any
) -> CartesianPos1D:
    """Self transform of 1D vectors."""
    return current


# TODO: use multi, with precedence
@dispatch(precedence=1)
def vconvert(
    target: type[RadialPos], current: RadialPos, /, **kwargs: Any
) -> RadialPos:
    """Self transform of 1D vectors."""
    return current


@dispatch.multi(
    (type[CartesianVel1D], CartesianVel1D, AbstractPos),
    (type[RadialVel], RadialVel, AbstractPos),
)
def vconvert(
    target: type[AbstractVel1D],
    current: AbstractVel1D,
    position: AbstractPos,
    /,
    **kwargs: Any,
) -> AbstractVel1D:
    """Self transform of 1D Velocities."""
    return current


# Special-case where self-transform doesn't need position
@dispatch.multi(
    (type[CartesianVel1D], CartesianVel1D),
    (type[RadialVel], RadialVel),
)
def vconvert(
    target: type[AbstractVel1D], current: AbstractVel1D, /, **kwargs: Any
) -> AbstractVel1D:
    """Self transform of 1D Velocities."""
    return current


@dispatch.multi(
    (
        type[CartesianAcc1D],
        CartesianAcc1D,
        AbstractVel,
        AbstractPos,
    ),
    (type[RadialAcc], RadialAcc, AbstractVel, AbstractPos),
)
def vconvert(
    target: type[AbstractAcc1D],
    current: AbstractAcc1D,
    velocity: AbstractVel,
    position: AbstractPos,
    /,
    **kwargs: Any,
) -> AbstractAcc1D:
    """Self transform of 1D Accs."""
    return current


# Special-case where self-transform doesn't need velocity or position
@dispatch.multi(
    (type[CartesianAcc1D], CartesianAcc1D),
    (type[RadialAcc], RadialAcc),
)
def vconvert(
    target: type[AbstractAcc1D],
    current: AbstractAcc1D,
    /,
    **kwargs: Any,
) -> AbstractAcc1D:
    """Self transform of 1D Accs."""
    return current


# =============================================================================
# CartesianPos1D


@dispatch
def vconvert(
    target: type[RadialPos], current: CartesianPos1D, /, **kwargs: Any
) -> RadialPos:
    """CartesianPos1D -> RadialPos.

    The `x` coordinate is converted to the radial coordinate `r`.
    """
    return target(r=current.x)


# =============================================================================
# RadialPos


@dispatch
def vconvert(
    target: type[CartesianPos1D], current: RadialPos, /, **kwargs: Any
) -> CartesianPos1D:
    """RadialPos -> CartesianPos1D.

    The `r` coordinate is converted to the `x` coordinate of the 1D system.
    """
    return target(x=current.r.distance)


# =============================================================================
# CartesianVel1D


@dispatch
def vconvert(
    target: type[CartesianVel1D], current: CartesianVel1D, /
) -> CartesianVel1D:
    """CartesianVel1D -> CartesianVel1D with no position.

    Cartesian coordinates are an affine coordinate system and so the
    transformation of an n-th order derivative vector in this system do not
    require lower-order derivatives to be specified. See
    https://en.wikipedia.org/wiki/Tensors_in_curvilinear_coordinates for more
    information. This mixin provides a corresponding implementation of the
    `coordinax.vconvert` method for Cartesian velocities.

    Examples
    --------
    >>> import coordinax as cx
    >>> v = cx.vecs.CartesianVel1D.from_([1], "m/s")
    >>> cx.vconvert(cx.vecs.CartesianVel1D, v) is v
    True

    """
    return current


# =============================================================================
# CartesianAcc1D


@dispatch
def vconvert(
    target: type[CartesianAcc1D], current: CartesianAcc1D, /
) -> CartesianAcc1D:
    """CartesianAcc1D -> CartesianAcc1D with no position.

    Cartesian coordinates are an affine coordinate system and so the
    transformation of an n-th order derivative vector in this system do not
    require lower-order derivatives to be specified. See
    https://en.wikipedia.org/wiki/Tensors_in_curvilinear_coordinates for more
    information. This mixin provides a corresponding implementation of the
    `coordinax.vconvert` method for Cartesian vectors.

    Examples
    --------
    >>> import coordinax as cx
    >>> a = cx.vecs.CartesianAcc1D.from_([1], "m/s2")
    >>> cx.vconvert(cx.vecs.CartesianAcc1D, a) is a
    True

    """
    return current


###############################################################################
# Corresponding Cartesian classes


@dispatch
def cartesian_vector_type(
    obj: type[AbstractPos1D] | AbstractPos1D, /
) -> type[AbstractPos1D]:
    """Return the corresponding Cartesian vector class."""
    return CartesianPos1D


@dispatch
def cartesian_vector_type(
    obj: type[AbstractVel1D] | AbstractVel1D, /
) -> type[AbstractVel1D]:
    """Return the corresponding Cartesian vector class."""
    return CartesianVel1D


@dispatch
def cartesian_vector_type(
    obj: type[AbstractAcc1D] | AbstractAcc1D, /
) -> type[AbstractAcc1D]:
    """Return the corresponding Cartesian vector class."""
    return CartesianAcc1D


###############################################################################
# Corresponding time derivative classes

# -----------------------------------------------
# Position -> Velocity


@dispatch
def time_derivative_vector_type(
    obj: type[CartesianPos1D] | CartesianPos1D, /
) -> type[CartesianVel1D]:
    """Return the corresponding time derivative class."""
    return CartesianVel1D


@dispatch
def time_derivative_vector_type(obj: type[RadialPos] | RadialPos, /) -> type[RadialVel]:
    """Return the corresponding time derivative class."""
    return RadialVel


# -----------------------------------------------
# Velocity -> Position


@dispatch
def time_antiderivative_vector_type(
    obj: type[CartesianVel1D] | CartesianVel1D, /
) -> type[CartesianPos1D]:
    """Return the corresponding time antiderivative class."""
    return CartesianPos1D


@dispatch
def time_antiderivative_vector_type(
    obj: type[RadialVel] | RadialVel, /
) -> type[RadialPos]:
    """Return the corresponding time antiderivative class."""
    return RadialPos


# -----------------------------------------------
# Velocity -> Acceleration


@dispatch
def time_derivative_vector_type(
    obj: type[CartesianVel1D] | CartesianVel1D, /
) -> type[CartesianAcc1D]:
    """Return the corresponding time derivative class."""
    return CartesianAcc1D


@dispatch
def time_derivative_vector_type(obj: type[RadialVel] | RadialVel, /) -> type[RadialAcc]:
    """Return the corresponding time derivative class."""
    return RadialAcc


# -----------------------------------------------
# Acceleration -> Velocity


@dispatch
def time_antiderivative_vector_type(
    obj: type[CartesianAcc1D] | CartesianAcc1D, /
) -> type[CartesianVel1D]:
    """Return the corresponding time antiderivative class."""
    return CartesianVel1D


@dispatch
def time_antiderivative_vector_type(
    obj: type[RadialAcc] | RadialAcc, /
) -> type[RadialVel]:
    """Return the corresponding time antiderivative class."""
    return RadialVel

==> coordinax-0.22-py3-none-any.whl: coordinax/_src/vectors/d1/register_vectorapi.py <==
"""Representation of coordinates in different systems."""

__all__: list[str] = []


from plum import dispatch

import coordinax._src.vectors.custom_types as ct
from .base import AbstractAcc1D, AbstractPos1D, AbstractVel1D
from .cartesian import CartesianAcc1D, CartesianPos1D, CartesianVel1D
from .radial import RadialAcc, RadialPos, RadialVel
from coordinax._src.vectors.private_api import combine_aux

###############################################################################
# Vector Transformation

# =============================================================================
# `vconvert_impl`


@dispatch
def vconvert(
    to_vector: type[CartesianPos1D],
    from_vector: type[RadialPos],
    params: ct.ParamsDict,
    /,
    *,
    in_aux: ct.OptAuxDict = None,
    out_aux: ct.OptAuxDict = None,
    units: ct.OptUSys = None,
) -> tuple[ct.ParamsDict, ct.AuxDict]:
    """RadialPos -> CartesianPos1D.

    The `r` coordinate is converted to the `x` coordinate of the 1D system.
    """
    return {"x": params["r"]}, combine_aux(in_aux, out_aux)


@dispatch
def vconvert(
    to_vector: type[RadialPos],
    from_vector: type[CartesianPos1D],
    p: ct.ParamsDict,
    /,
    *,
    in_aux: ct.OptAuxDict = None,
    out_aux: ct.OptAuxDict = None,
    units: ct.OptUSys = None,
) -> tuple[ct.ParamsDict, ct.AuxDict]:
    """CartesianPos1D -> RadialPos.

    The `x` coordinate is converted to the `r` coordinate of the 1D system.
    """
    return {"r": p["x"]}, combine_aux(in_aux, out_aux)


###############################################################################
# Corresponding Cartesian classes


@dispatch
def cartesian_vector_type(
    obj: type[AbstractPos1D] | AbstractPos1D, /
) -> type[AbstractPos1D]:
    """Return the corresponding Cartesian vector class."""
    return CartesianPos1D


@dispatch
def cartesian_vector_type(
    obj: type[AbstractVel1D] | AbstractVel1D, /
) -> type[AbstractVel1D]:
    """Return the corresponding Cartesian vector class."""
    return CartesianVel1D


@dispatch
def cartesian_vector_type(
    obj: type[AbstractAcc1D] | AbstractAcc1D, /
) -> type[AbstractAcc1D]:
    """Return the corresponding Cartesian vector class."""
    return CartesianAcc1D


###############################################################################
# Corresponding time derivative classes

# -----------------------------------------------
# Position -> Velocity


@dispatch
def time_derivative_vector_type(
    obj: type[CartesianPos1D] | CartesianPos1D, /
) -> type[CartesianVel1D]:
    """Return the corresponding time derivative class."""
    return CartesianVel1D


@dispatch
def time_derivative_vector_type(obj: type[RadialPos] | RadialPos, /) -> type[RadialVel]:
    """Return the corresponding time derivative class."""
    return RadialVel


# -----------------------------------------------
# Velocity -> Position


@dispatch
def time_antiderivative_vector_type(
    obj: type[CartesianVel1D] | CartesianVel1D, /
) -> type[CartesianPos1D]:
    """Return the corresponding time antiderivative class."""
    return CartesianPos1D


@dispatch
def time_antiderivative_vector_type(
    obj: type[RadialVel] | RadialVel, /
) -> type[RadialPos]:
    """Return the corresponding time antiderivative class."""
    return RadialPos


# -----------------------------------------------
# Velocity -> Acceleration


@dispatch
def time_derivative_vector_type(
    obj: type[CartesianVel1D] | CartesianVel1D, /
) -> type[CartesianAcc1D]:
    """Return the corresponding time derivative class."""
    return CartesianAcc1D


@dispatch
def time_derivative_vector_type(obj: type[RadialVel] | RadialVel, /) -> type[RadialAcc]:
    """Return the corresponding time derivative class."""
    return RadialAcc


# -----------------------------------------------
# Acceleration -> Velocity


@dispatch
def time_antiderivative_vector_type(
    obj: type[CartesianAcc1D] | CartesianAcc1D, /
) -> type[CartesianVel1D]:
    """Return the corresponding time antiderivative class."""
    return CartesianVel1D


@dispatch
def time_antiderivative_vector_type(
    obj: type[RadialAcc] | RadialAcc, /
) -> type[RadialVel]:
    """Return the corresponding time antiderivative class."""
    return RadialVel
